  rm(list=ls())
  # load the libary
  library(IsoMSFT);
  #-------------------------------Start of Setting----------------------------------------------
  dataFolder<-"./input/"
  dataSaveFolder<-"./output/"
  dataSaveFolderDebug<-paste(dataSaveFolder,"debug/",sep="")

  isFT<-TRUE               # otherwise TOF
  # rt, mz & int window
  rtWinMin<-120
  rtWinMax<-1500
  mzWinMin<-260            # also used in grouping to define mz bins
  mzWinMax<-800            # also used in grouping to define mz bins
  intThreshold<-2e+3

  if (isFT==TRUE)
  {
     # remove bad spectra
     minResRatio<-0.15

     # parameters used in removeHomoPeaks
     mzWinHomo<-0.25                      # m_z window to filter the homogeneous peak
     mzSubWinHomo<-c(0.02,0.06,0.125);
     intSubRatioHomo<-c(0.12,0.08,0.02);   # minimum ratio [ intensity to the maximum intensity within defined sub m_z window]
     maxMZDiffHomo<-0.1                   # m_z distance to define a solitary peak
  } else {
     # remove tailing
     intBase<- 1000
     intValidThreshold<-0.05;
     mzWinTail<-1
  }

  # parameters used in findPeakPairsDupl.R
  isFindPeakPairs<-TRUE                           # if FALSE, will read the csv file saved from previous run [used for test]
  chargeVec<-c(1.00336,0.50168,0.33445,0.25084);  # mz-dist vector to defined charge-types [1,2,3,4]
  mzTolChargeVec<-c(10e-6,10e-6,10e-6,10e-6);   # mz tolerance used to define a charge-type [chargeType=1,2,3,4]

  #intensity ratio range for valid 1st isotopic of C12 [could be different for different charge-types]
  intRatioRangeChargeMat<-matrix(c(0.12,0.4,0.25,0.5,0.3,0.55,0.35,0.6),2,4); # different cols ==> different chargeType
  intSaturated<-1e+8                              # saturated intensity
  intRatioC12_1stTag<-3;                          # the "max" intensity ratio of [intensity of C12 ] over [ the intensity of its first possible tag]
  intRatioC13_to_1stISO<-c(0.5,0.5,0.5,0.5);      # the maximum intensity ratio of valid C13 over that of the 1st isotope of C12
                                                  # for different tag
  intRatio1stIso_C13_C12<-c(0.5,2.0);             # the ratio range of [intensity ratio of 1st isotope over C13 ] over [intensity ratio of 1st isotope over C12],
                                                  # used to define lev1-peakpair
  mzTolIso<-c(10e-6,10e-6,10e-6,10e-6);               # mz tolerance used to search isotopes [more details of isotopes in findIsotopes.R]
  mzTolFixLev3<-20e-6;                             # mz_tolerance used to fix lev3 peak
  #Lev4 pairs mz distance range is hard coded in findPeakPairMZDist.R    [**********]

  # remove dimer
  mzTolDimer<-20e-6

  # parameters used in cleanDerivative.R
  mz_tol_dt<-20E-6

  # parameters used in deconvPeaks.R
  H_deconv<-1.007276
  mz_tol_deconv<-20E-6
  
  # parameters used in groupPairs.R
  gpMethod<-2
  if (gpMethod==1)
  {
     # use groupPairs(pkList,mzTol=5e-6,intRatioPeakOverValley=2)
     mzTolGroup<-20e-6;                              # same mzC group: abs(MZ-mzC)/mzC < mzTolGroup
     intRatioPeakOverValley=3.0;                      # the min intensity ratio of a good peak over the adjacent valley
  } else {
     # use groupPairsII(pkList,mzWin=1, mzTol=75e-6)
     mzWinGroup<-1
     mzTolGroup<-20e-6
  }

  # remove mz background
  isMZBackground<-TRUE       # if is FALSE, will use mzFilter.R
  mzTolBackground<-20e-6
  mzFileBackground<-"mzBackground.csv"

  # mz filter  for with given charge-tag type
  mzTolFilter<-20e-6
  mzFilterFile<-"mzFilter.csv"

  # remove Lighter pairs [only lev1,2,3? because no chargeType and tagType available for lev4]
  mzCutoffFile<-"mzCutoff.csv";                   # mz-cutoff list file name used in mzCutoff.R
  
  # filter 2charge-2tag
  mz_tol_ctfilter<-20e-6

  # vaild intensity ratio [light:heavy]filter
  minIntensity<-1e+4
  minRatio<-0.1
  maxRatio<-10
  if (isFT==TRUE) {minSN<-20;}
  #-------------------------------End of Setting----------------------------------------------

  # creating saving folder
  if ( ! file.exists(dataSaveFolder) )
  {
     dir.create(dataSaveFolder,showWarnings=FALSE, recursive=TRUE)
  }
  # creating saving folder for debug files
  if ( ! file.exists(dataSaveFolderDebug))
  {
     dir.create(dataSaveFolderDebug,showWarnings=FALSE, recursive=TRUE)
  }

  con <- file(paste(dataSaveFolderDebug,"screenlog.txt",sep=""));
  sink(con, append=TRUE); 
   
  # list all csv file
  csvfiles<-list.files(path=dataFolder,pattern="(.csv)$");
  for ( nf in 1:length(csvfiles) ) {
      # read data from each csv file
      SPFileFromDA<-csvfiles[nf]
      print("");
      print(paste("Processing ",SPFileFromDA))
 
      if (isFT==TRUE)
      {
         badSpectrumLog<-paste(dataSaveFolderDebug,"badSpectrumLog_",SPFileFromDA,sep="")
         pairsOnlyFewTimes<-paste(dataSaveFolder,"pairsOnlyShowFewTimes_",SPFileFromDA,sep="")
         finalResultFileName<-paste(dataSaveFolder,"IsoMSFT_results_",SPFileFromDA,sep="")
      } else {
         pairsOnlyFewTimes<-paste(dataSaveFolder,"pairsOnlyShowFewTimes_",SPFileFromDA,sep="")
         finalResultFileName<-paste(dataSaveFolder,"IsoMSTOF_results_",SPFileFromDA,sep="")
      }
      
      # read the data table exported from DataAnalysis
      oriCSVFile<-paste(dataFolder,SPFileFromDA,sep='')
      print(paste("Reading file: ",oriCSVFile))
      pk<-read.csv(oriCSVFile,header=T,sep=",")
      if (nrow(pk)==0) {stop("no data in original csv file");}

      # mz, rt window and intensity threshhold filtering
      indValid<-which(pk$rt>rtWinMin & pk$rt<rtWinMax & pk$mz>mzWinMin & pk$mz<mzWinMax & pk$maxo>intThreshold)
      pk<-pk[indValid,];
      if (nrow(pk)==0) {stop("no data after mz, rt window and intensity threshhold filtering");}

      if (isFT==TRUE)
      {
         #filter bad spectra
         pk<-removeBadSpectra(pk,minResRatio,badSpectrumLog);
         print(paste("Before removing HomoPeaks Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
         pk<-removeHomoPeaks(pk,mzWinHomo,mzSubWinHomo,intSubRatioHomo,maxMZDiffHomo)
      } else {
         badSpectrumLog<-NA
         # remove tailing
         pk<-removeTailing(pk,intBase,intValidThreshold,mzWinTail)
      }
      if (nrow(pk)==0) {stop("no data before find PeakPair!!");}
      
      # find peak pairs
      print(paste("Start findPeakPairs Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      pairs<-findPeakPairsDupl(pk,chargeVec,mzTolChargeVec,intRatioRangeChargeMat,intSaturated,intRatioC12_1stTag,intRatioC13_to_1stISO,intRatio1stIso_C13_C12,
                               mzTolIso,mzTolFixLev3,isFT);
      if (nrow(pairs)==0) {stop("no data after findPeakPairs");}
      print(paste("After findPeakPairs Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      rm(pk)

      # remove dimer
      print('');
      nodimerPairs<-removeDimer(pairs,mzTolDimer); rm(pairs)
      if (nrow(nodimerPairs)==0) {stop("no data after removeDimer");}
      
      # clean derivatives
      print('');
      print(paste("Start cleanDerivative Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      nodtPairs<-cleanDerivatives(nodimerPairs,mz_tol_dt); rm(nodimerPairs)
      if (nrow(nodtPairs)==0) {stop("no data after cleanDerivatives");}

      # deconv 
      print('');
      print(paste("Start deconvPeaks Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      deconvPairs<-deconvPeaks(nodtPairs,H_deconv,mz_tol_deconv); rm(nodtPairs)
      if (nrow(deconvPairs)==0) {stop("no data after deconvPairs");}
      
      # grouping ==> removing the tailing ones
      print('');
      print(paste("Before grouping Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      if (gpMethod==1)
      {
         gpPairs<-groupPairs(deconvPairs,mzTolGroup,intRatioPeakOverValley,badSpectrumLog);
      } else {
         gpPairs<-groupPairsII(deconvPairs,mzWinMin,mzWinMax,mzWinGroup,mzTolGroup,badSpectrumLog,NA,pairsOnlyFewTimes);
      }
      rm(deconvPairs)
      if (nrow(gpPairs)==0) {stop("no data after grouping");}

      # filter the background mz
      if (isMZBackground==TRUE)
      { 
         print('');
         print(paste("Start mzFilterBackground Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
         gpPairs<-mzFilterBackground(gpPairs,mzFileBackground,mzTolBackground);
      } else {
         # use mzFilter.R
         print('');
         if (file.exists(mzFilterFile))
         {
            print(paste("Start mzFilter Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
            gpPairs<-mzFilter(gpPairs,mzFilterFile,mzTolFilter);
            print(paste("After mzFilter Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
         } else {
            print(paste("warning: mzFilterFile is not found! ==> ",mzFilterFile))
         }
      }

      # remove pairs with a lighter mz
      if (file.exists(mzCutoffFile))
      {
         print('');
         print(paste("Start remove lighter mz pairs Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
         gpPairs<-mzCutoff(gpPairs,mzCutoffFile);
         print(paste("After mzCutoff Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      }

      # filtering based on ratio & intensity 
      indFiltered<-which(gpPairs$int_light>=minIntensity & gpPairs$ratio_light_to_heavy>=minRatio & gpPairs$ratio_light_to_heavy<=maxRatio)
      pk.flt<-gpPairs[indFiltered, ]; rm(gpPairs)
      pk.flt<-pk.flt[order(pk.flt$nCharge,pk.flt$nTag,decreasing=FALSE),]

      # filter 2charge-2tag
      pk.flt<-ctFilter(pk.flt,mz_tol_ctfilter)
            
      # format output
      # sample, scanNum, rt, mz_light, mz_heavy, distance, int_light, int_heavy, ratio_light_to_heavy [,sn_light, sn_heavy], nCharge, nTag, pairLev, isSaturated
      nLen<-nrow(pk.flt);
      pkFormated<-vector(mode="character",length=(nLen+1));
      if (isFT==TRUE)
      {
        pkFormated[1]<-"sample,scanNum,rt,mz_light,mz_heavy,distance,int_light,int_heavy,ratio_light_to_heavy, sn_light, sn_heavy, nCharge, nTag, pairLev,isSaturated"
        pkFormated[2:(nLen+1)]<-sprintf("%2d, %5d, %0.2f, %0.6f, %0.6f, %0.6f, %0.2E, %0.2E, %0.2f, %0.4f, %0.4f, %2d, %2d, %2d, %2d",pk.flt[,1],pk.flt[,2],pk.flt[,3],pk.flt[,4],pk.flt[,5],pk.flt[,6],pk.flt[,7],pk.flt[,8],pk.flt[,9],pk.flt[,10],pk.flt[,11],pk.flt[,12],pk.flt[,13],pk.flt[,14],pk.flt[,15])
      } else {
        pkFormated[1]<-"sample,scanNum,rt,mz_light,mz_heavy,distance,int_light,int_heavy,ratio_light_to_heavy, nCharge, nTag, pairLev,isSaturated"
        pkFormated[2:(nLen+1)]<-sprintf("%2d,%5d, %0.2f, %0.6f, %0.6f, %0.6f, %0.2E, %0.2E, %0.2f, %2d, %2d, %2d, %2d",pk.flt[,1],pk.flt[,2],pk.flt[,3],pk.flt[,4],pk.flt[,5],pk.flt[,6],pk.flt[,7],pk.flt[,8],pk.flt[,9],pk.flt[,10],pk.flt[,11],pk.flt[,12],pk.flt[,13])
      }
      write.table(pkFormated, file=finalResultFileName,col.names=FALSE,row.names=FALSE,quote=FALSE);
      
      print(paste("Done File: ",SPFileFromDA," Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
      print('');
      rm(SPFileFromDA)
  }
  print(paste("Done everything Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
  sink();
