##############################
# University of Alberta
#          Apr 2013   added the adListFile
#############################

cleanDerivatives<- function(peaklist,mz_tol=20E-6,isDebug=FALSE)
{
  dtListFile<-"derivatives.csv"
  if (!file.exists(mzFilterFile))
  {
     stop(paste(adListFile," is not found in current folder",sep=""))
  }
  dtList<-read.csv(dtListFile,header=F,sep=",")
  numDT<-nrow(dtList)
  print(paste("  cleanDerivatives.R: mz_tol=",mz_tol))
  peaklist<-as.data.frame(peaklist);       
 # check if scan number exists
 if (length(which(colnames(peaklist)=="scanNum"))==0)
 {
    colnames(peaklist)
    stop("cleanDerivatives is scan-number based, but no scan number field [scanNum] is found from input")
 }

 # create the sample column if necessary
 if (length(which(colnames(peaklist)=="sample"))==0)
 {
   peaklist$sample <- 1;
   samples<-1;
 } else {
   peaklist$sample <- as.factor(peaklist$sample);
   samples<-levels(peaklist$sample);
 }

 # sort all the peaks first if they are not already sorted
 # note: test only on light peaks will work for both since they are paired 
 # scan number is heritage from the lighter peak in findpeakpairs
 inx.ord<- order(peaklist$sample, peaklist$scanNum, peaklist$mz_light, decreasing=F);
 peaklist.sorted <- peaklist[inx.ord, ]; rm(peaklist);

 # add a global unique id (row number) to tag each row
 numRowPK<-nrow(peaklist.sorted);
 peaklist.sorted$id<-1:numRowPK;
 adduct.vec <- vector(mode="logical", length = numRowPK);
 
 adduct.count = 0;
 if (isDebug==TRUE)
 {
    cat(paste("#", "mz A", "mz B", "rt A", "rt B", sep="\t"), "\n");
 }

 for (s in 1:length(samples)){
     current.sample <- peaklist.sorted[which(peaklist.sorted$sample==samples[s]), ];
     numRowC<-nrow(current.sample);
     isAdductC<-vector(mode='logical',length=numRowC);
     #isSkip<-vector(mode='logical',length=numRowC); # removed May 19, 2013
     m <- 0;
     # iterate through all peaks
     while ( m < (numRowC-1))
     {
           m <- m+1;
           mzA<-current.sample$mz_light[m];
           scanNumA<-current.sample$scanNum[m];
           
           # need check this, should?
           #if (isAdductC[m] || isSkip[m])
           if (isAdductC[m])
           {
              next;
           }     
           # compare with all the remaining ones
           for (n in (m+1):numRowC)
           {

               if (current.sample$scanNum[n]!=scanNumA)
               {
                  # should with same scan-number
                  # if sorted correctly, all scan-numbers after should be greater than scanNumA
                  break;
               }

               # if with same scan-number and not be used, then compare mz distance
               mzB<-current.sample$mz_light[n];

               if (mzB<mzA)
               {
                  stop("cleanadducts: should not see this msg ==> negative mz dist is found!!!")
               } 

               dist <-mzB-mzA;
               mzDiff<-mzA*mz_tol
               for (ntype in 1:numDT)
               {
                   if (dist>dtList$V2[ntype]-mzDiff & dist < dtList$V2[ntype]+mzDiff)
                   {
                      adduct.count = adduct.count + 1;
                      # get the global id for the current row
                      if (dtList$V3[ntype]=="A")
                      {
                        # Na, K NH4 ...
                        isAdductC[n]<-TRUE;
                        adduct.vec[current.sample$id[n]]<-TRUE
                      } else if (dtList$V3[ntype]=="N") {
                        # OH, H2O, SH, CO2, HCO2H
                        isAdductC[m]<-TRUE;
                        adduct.vec[current.sample$id[m]]<-TRUE
                        #isSkip[n]<-TRUE;
                      } else {
                        stop(paste("unknow ad type: ",adList$V3[ntype],sep=""))
                      }
                      if (isDebug==TRUE)
                      {
                         rtA<-current.sample$rt_light[m];
                         rtB<-current.sample$rt_light[n];
                         cat(paste(adduct.count, mzA, mzB, rtA, rtB, sep="\t"), "\n");
                      }
                      break; # should be only one-type  
                   } # if falls in
               } # type of derivatives
           } # end of n 
     }  # end of m
 } # end of samples

 print(paste("  cleanDerivatives.R: a total of ", adduct.count, " pairs removed!"));
 peaklist.sorted$id<-NULL;
 return(peaklist.sorted[!adduct.vec, ]);
}
