##############################
# Jeff Xia, June, 2009
# University of Alberta
# xianmin  Jan 2012   optimized and scan number based 
#                     must have the scan number column [scanNum]
#############################

deconvPeaks<-function(peaklist,H=1.007276,mz_tol=30E-6,isDebug=FALSE){

        print(paste("  deconvPeaks.R:          H=",H))
        print(paste("                     mz_tol=",mz_tol))

        peaklist<-as.data.frame(peaklist);
        # check if scan number exists
        if (length(which(colnames(peaklist)=="scanNum"))==0)
        {
           colnames(peaklist)
           stop("deconvpeaks is scan-number based, but no scan number field [scanNum] is found from input")
        }
        # create the sample column if necessary
        if (length(which(colnames(peaklist)=="sample"))==0)
        {
          peaklist$sample <- 1;
          samples <- 1;
        } else {
          peaklist$sample <- as.factor(peaklist$sample);
          samples <- levels(peaklist$sample);
        }

        # sort all the peaks first if they are not already sorted
        # note: scan-number is heritaged from the lighter one, only on light peaks will work for both since they are paired 
        inx.ord<- order(peaklist$sample, peaklist$scanNum, peaklist$mz_light, decreasing=F);
        peaklist.sorted <- peaklist[inx.ord, ]; rm(peaklist);

        numRowPK<-nrow(peaklist.sorted);
        # add a global unique id (row number) to tag each row
        peaklist.sorted$id<-1:numRowPK;
        deconv.vec <- vector(mode="logical", length = numRowPK);

        deconv.count = 0;
        if (isDebug==TRUE)
        {
           cat(paste("#", "mz A", "mz B", "rt A", "rt B", sep="\t"), "\n");
        }

        for ( s in 1:length(samples)){
            current.sample <- peaklist.sorted[which(peaklist.sorted$sample==samples[s]), ];
            m <- 0;
            numRowC<-nrow(current.sample);
            isDeconvC<-vector(mode='logical',length=numRowC);

            # iterate through all peaks
            while (m < (numRowC-1)){
                    m <- m+1;

                    # need check this, should?
                    if (isDeconvC[m]==TRUE)
                    {
                        next;
                    }
                    mzA<-current.sample$mz_light[m];
                    scanNumA<-current.sample$scanNum[m];

                    # compare with all the remaining ones
                    for ( n in (m+1):numRowC)
                    {
                        if (current.sample$scanNum[n]!=scanNumA)
                        {
                           # should with same scan-number
                           # if sorted correctly, all scan-numbers after should be greater than scanNumA
                           break;
                        }


                        # if with same scan-number, then compare mz distance
                        mzB<-current.sample$mz_light[n];
                        if (mzB<mzA)
                        {
                           stop("deconvPeaks.R: should not see this msg ==> negative mz dist is found!!!")
                        }
                        dist <-mzB-mzA;

                        # double charged or dimer
                        if ( mzB > 2*mzA-H-mzA*mz_tol & mzB < 2*mzA-H + mzA*mz_tol )
                        {
                           deconv.count = deconv.count + 1;

                           # remove peak with lower intensity since we cannot tell dimer/double charges
                           intA<-current.sample$int_light[m];        
                           intB<-current.sample$int_light[n];
                           if (intA>intB)
                           {
                              row.id<-current.sample[n,"id"]; 
                           } else {
                              row.id<-current.sample[m,"id"]; 
                           }
                           if (isDebug==TRUE)
                           {
                              rtA<-current.sample$rt_light[m];
                              rtB<-current.sample$rt_light[n];
                              cat(paste(deconv.count, mzA, mzB, rtA, rtB, sep="\t"), "\n");
                           }
                           deconv.vec[row.id]<-TRUE;
                           isDeconvC[n]<-TRUE
                        } else if (mzB > 3*mzA-2*H-mzA*mz_tol & mzB < 3*mzA-2*H + mzA*mz_tol) { # trible charged
                           deconv.count = deconv.count + 1;
                           # should we take the higher intensity one??
                           current.row.id<-current.sample[n,"id"];
                           if (isDebug==TRUE)
                           {
                              rtA<-current.sample$rt_light[m];
                              rtB<-current.sample$rt_light[n];
                              cat(paste(deconv.count, mzA, mzB, rtA, rtB, sep="\t"), "\n");
                           }
                           deconv.vec[current.row.id]<-TRUE;
                           isDeconvC[n]<-TRUE
                        }
                    } 
            }
        }

        print(paste("  deconvPeaks.R: a total of ", deconv.count, " pairs removed!"));
        peaklist.sorted$id<-NULL;
        return(peaklist.sorted[!deconv.vec, ]);
}

