# group pairs with same charge number and tag number
#
# Usage:
#     newPK <-groupPairsII(oldPairList,mzWinMin,mzWinMax,mzWin,mzTol,badSpectraFile,debugFile)
#                   mzWin                   : define the mz-window
#                   mzTol                   : mz tolerance used to define a mz-subbin within a "continual" scan Nos
#                badSpectraFile             : potential bad spectra information
#                debugFile                  : save the debug information
#               pairsOnlyFewTime            : log file for those pairs showing up just in a limited scanNos
# Update:
#     May20, 2013:  first code
#     June3, 2013:  consider all 75ppm sub-mz groups

groupPairsII<-function(pkList,mzWinMin=200,mzWinMax=800,mzWin=1,mzTol=75e-6,badSpectraFile=NA,debugFile=NA,pairsOnlyFewTime="paisOnlyFewTime.csv")
{

     isDebug<-FALSE;
     if (length(debugFile)!=0 && !is.na(debugFile))
     {
        isDebug<-TRUE;
     }

     #order charge number, then tag number
     pkList<-as.data.frame(pkList);
     numRow<-nrow(pkList);

     pkListSorted<-pkList[order(pkList$nCharge,pkList$nTag), ]; rm(pkList)
     isGoodPairs<-vector(mode='logical',length=numRow);
     if (isDebug==TRUE)
     {
        print(paste("groupPairsII: mzWinMin=",mzWinMin," mzWinMax=",mzWinMax," mzWin=",mzWin))
        mzBinVec<-vector(mode='integer',length=numRow);
        noIDInMzBinVec<-mzBinVec;
        mzSubWin75ppmVec<-mzBinVec;
     }
     isGoodPairsType<-vector(mode='integer',length=numRow);
     
     chargeVec<-pkListSorted$nCharge;
     tagVec<-pkListSorted$nTag;
     intVec<-pkListSorted$int_light;
     mzVec<-pkListSorted$mz_light;
     isSTVec<-pkListSorted$isSaturated;

     scanNoVec<-pkListSorted$scanNum;
     oriIndex<-c(1:numRow);
     chargeUq<-unique(chargeVec);
     tagUq<-unique(tagVec);
     numTolChargeType<-length(chargeUq);
     numTolTagType<-length(tagUq);

     maxScanMissing<-3

     # read bad spectra if need
     if (is.na(badSpectraFile))
     {
        badSpectra<-NA;
     } else {
       badSpectra<-scan(badSpectraFile,quiet=TRUE);
     }
     #
     for (nC in 1:numTolChargeType)
     {
         for (nT in 1:numTolTagType)
         {
             if (chargeUq[nC]!=0 && tagUq[nT]==0) { next; }
             if (chargeUq[nC]==0 && tagUq[nT]!=0) { break; }
             # find pairs with same chargeType and tagType
             indexSCT<-which(chargeVec==chargeUq[nC] & tagVec==tagUq[nT]);
             if (length(indexSCT)<1) { next; }
             mzCurrent<-mzVec[indexSCT];

             # sorted by mz, break into sub-mz-bins
             mzSortInfo<-sort(mzCurrent,index.return=TRUE)
             mzCurrentSort<-mzSortInfo$x
             noCurrentSort<-scanNoVec[indexSCT[mzSortInfo$ix]]
             intCurrentSort<-intVec[indexSCT[mzSortInfo$ix]]
             oriIndCurrentSort<-oriIndex[indexSCT[mzSortInfo$ix]]

             mz0 <- max(mzWinMin,floor(mzCurrentSort[1]))
             numMZ<-length(mzCurrentSort)
             maxMZ<-min(ceiling(mzCurrentSort[numMZ]),mzWinMax)
             mz1<-mz0+mzWin
             isFirst<-TRUE
             mzBinId<-0
             while  (mz1<=maxMZ)
             {
                #if (isDebug==TRUE) {print(paste("mz0: ",mz0," mz1: ",mz1))}
                if (isFirst==TRUE)
                {
                   indMZWin<-which(mzCurrentSort>=mz0 & mzCurrentSort<=mz1)
                   isFirst<-FALSE
                } else {
                   indMZWin<-which(mzCurrentSort>mz0 & mzCurrentSort<=mz1)
                }

                if (length(indMZWin)>0) 
                {
                  mzInWin<-mzCurrentSort[indMZWin]
                  noInWin<-noCurrentSort[indMZWin]
                  intInWin<-intCurrentSort[indMZWin]
                  oriInWin<-oriIndCurrentSort[indMZWin]
                  
                  # sorted by scan number
                  indNoInfo<-sort(noInWin,index.return=TRUE)
                  scanNoID<-labelScanNoID(indNoInfo$x,maxScanMissing,badSpectra)
                  scanNoIDuq<-unique(scanNoID)
                  numScanNoGP<-length(scanNoIDuq)
                  if (isDebug==TRUE)
                  {
                     mzBinId<-mzBinId+1
                     mzBinVec[oriInWin]<-mzBinId
                  }
                  
                  for (ns in 1:numScanNoGP)
                  {
                      indNoGPc<-which(scanNoID==scanNoIDuq[ns])       # find a scan-no group
                      intNoGPc<-intInWin[indNoInfo$ix[indNoGPc]]      # corresponding intensity
                      mzNoGPc<-mzInWin[indNoInfo$ix[indNoGPc]]        # corresponding mz
                      oriIndNoGPc<-oriInWin[indNoInfo$ix[indNoGPc]]   # corresponding original index
                      isUsed<-vector(mode='logical',length=length(indNoGPc))
                  
                      # how to define a good peak, find the highest intensity?
                      #print(paste("length intNoGPc:",length(intNoGPc)))
                      if (isDebug==TRUE) {n75ppm<-1;}
                      while (length(which(isUsed==FALSE))>1)
                      {

                         intMax<-max(intNoGPc[which(isUsed==FALSE)])
                         indIntMax<-which(intNoGPc==intMax & isUsed==FALSE)[1]  # added isUsed==FALSE to fixed a infinite looping bug, aug21, 2013 
                         mzAtIntMax<-mzNoGPc[indIntMax]

                         # mz-sub-window, 75ppm
                         indMZSubWin <- which( abs(mzNoGPc-mzAtIntMax)/mzTol<=mzAtIntMax & isUsed==FALSE)
                         if (length(indMZSubWin)==0) { stop("groupPairsII: should not see this msg!!"); }

                         isUsed[indMZSubWin]<-TRUE
                         intMZSubWin<-intNoGPc[indMZSubWin]
                         mzMZSubWin<-mzNoGPc[indMZSubWin]
                         noMZSubWin<-noInWin[indNoInfo$ix[indNoGPc[indMZSubWin]]]
                         
                         if (isDebug==TRUE)
                         {
                             noIDInMzBinVec[oriIndNoGPc]<-ns
                             mzSubWin75ppmVec[oriIndNoGPc[indMZSubWin]]<-n75ppm
                             n75ppm<-n75ppm+1;
                         }
                         # sort by scanNo again
                         if (length(indMZSubWin)<=1)
                         {
                            # ignore, no possibility to have more than 3 pairs in a segment
                            next;
                         } else {
                            noMZSubWinSortInfo<-sort(noMZSubWin,index.return=TRUE)
                            mzMZSubWinSort<-mzMZSubWin[noMZSubWinSortInfo$ix] #mz
                            oriIndxMZSubWinSort<-oriIndNoGPc[indMZSubWin[noMZSubWinSortInfo$ix]] # original index
                            # re-label scan-number
                            noGPIDMZSubWin<-labelScanNoID(noMZSubWinSortInfo$x,maxScanMissing) # no bad spectra
                            noGPIDMZSubWinUQ<-unique(noGPIDMZSubWin)
                            numGPC<-length(noGPIDMZSubWinUQ)
                            
                            for (nns in 1:numGPC)
                            {
                                indCGP<-which(noGPIDMZSubWin==noGPIDMZSubWinUQ[nns])
                                isSTCGP<-isSTVec[oriIndNoGPc[indCGP]]
                                if (length(which(isSTCGP==1))>0)
                                {
                                   # if any saturated, use only the saturated ones
                                   indCGP<-indCGP[which(isSTCGP==1)]
                                }
                                numCGP<-length(indCGP)
                                if (numCGP<=1)
                                {
                                   # ignore
                                   isGoodPairsType[oriIndxMZSubWinSort[indCGP]]<- -2
                                   next;
                                } else if (numCGP<=5) {
                                   #  choose the one with maximum intensity
                                   levCGP<-pkListSorted$pairLev[oriIndxMZSubWinSort[indCGP]]                                   
                                   indLev12<-which(levCGP==1 | levCGP==2)
                                   if (length(indLev12)>=1)
                                   {
                                      intCGP<-intMZSubWin[noMZSubWinSortInfo$ix[indCGP[indLev12]]]
                                      maxIntLoc<-which(intCGP==max(intCGP))
                                      if (length(maxIntLoc)>1) {print("warning: found more than two good paris in a seg [sub-mz window]")}
                                      isGoodPairs[oriIndxMZSubWinSort[indCGP[indLev12[maxIntLoc[1]]]]]<-TRUE
                                      if (isDebug==TRUE)  {isGoodPairsType[oriIndxMZSubWinSort[indCGP[indLev12[maxIntLoc]]]]<-1}
                                   } else {
                                      if (isDebug==TRUE) {isGoodPairsType[oriIndxMZSubWinSort[indCGP]]<- -1;}
                                   }
                                } else {
                                   # intensity-scanNo space, find the peaks
                                   intCGP<-intMZSubWin[noMZSubWinSortInfo$ix[indCGP]]
                                   levCGP<-pkListSorted$pairLev[oriIndxMZSubWinSort[indCGP]]
                                   indLev12<-which(levCGP==1 | levCGP==2)
                                   isLabelMaxInt<-length(indLev12)>=1
                                   peakIndex<-getPeakIndex(intCGP,FALSE)
                                   if (length(peakIndex)>0) {
                                      isGoodPairs[oriIndxMZSubWinSort[indCGP[peakIndex]]]<-TRUE
                                   } else if (isLabelMaxInt==TRUE) {
                                      peakIndex<-getPeakIndex(intCGP,isLabelMaxInt)
                                      isGoodPairs[oriIndxMZSubWinSort[indCGP[peakIndex]]]<-TRUE
                                   }
                                   
                                   if (isDebug==TRUE)  {isGoodPairsType[oriIndxMZSubWinSort[indCGP]]<-2}
                                }                                
                            } # Loop of sub-scan-no-groups                              
                         } # if condition of number of elements in a mz-sub-window
                      } # loop of all 75ppm group
                  } # Loop of scan-no-groups
               } # if found within current mz-bin
               #next bin
               mz0<-mz1
               mz1<-mz0+mzWin
             } # Loop of mz-bins
         } # tag-type
     } #  charge-type


     if (isDebug==TRUE)
     {
        pkListSorted$mzBinVec<-mzBinVec
        pkListSorted$noIDInMzBinVec<-noIDInMzBinVec
        pkListSorted$mzSubWin75ppmVec<-mzSubWin75ppmVec
        pkListSorted$isGoodPairsType<-isGoodPairsType
        pkListSorted$isGoodPairs<-isGoodPairs;
        write.table(pkListSorted,file=debugFile,col.name=TRUE,sep=",",row.name=FALSE);
        pkListSorted$isGoodPairs<-NULL;
        pkListSorted$isGoodPairsType<-NULL
        pkListSorted$mzSubWin75ppmVec<-NULL
        pkListSorted$noIDInMzBinVec<-NULL
        pkListSorted$mzBinVec<-NULL
     }
     # save the pairs showing only in few scanNos
     write.table(pkListSorted[which(isGoodPairsType<0),],file=pairsOnlyFewTime,col.name=TRUE,sep=",",row.name=FALSE);

     # return
     numPairRemove<-length(which(isGoodPairs==FALSE))
     save(file="tempfile.csv",list=ls())
     print(paste("groupPairsII.R: a total of  ",numPairRemove," pairs removed!"),sep="")
     return(pkListSorted[which(isGoodPairs==TRUE),]);
}

getPeakIndex<-function(intList,isMax)
{
  
  numInt<-length(intList)
  if (numInt<=5) {stop("Should have more than 5 in labelPeaks")}
  isPeak<-vector(mode="logical",length=numInt)
  intR_L<-vector(mode="numeric",length=numInt)
  intL_R<-intR_L
  intR_L<-intList[2:numInt]-intList[1:(numInt-1)]
  intL_R[2:numInt]<-intList[1:(numInt-1)]-intList[2:numInt]

  for (nn in 3:(numInt-2))
  {
     if (intL_R[nn]<0 && intL_R[nn-1]<0)
     {
        if (intR_L[nn]<0 && intR_L[nn+1]<0)
        {
           isPeak[nn]<-TRUE
        }
     }
  }

  if (isMax==TRUE)
  {
     # "must" contains max-int-location
     maxInt<-max(intList)
     isPeak[intList==maxInt]<-TRUE
  }

  indtmp<-c(1:numInt)
  indTrue<-which(isPeak==TRUE)
  pkindex<-indtmp[indTrue]
  return (pkindex)
}
labelScanNoID<-function(scanNoList,maxScanMissing,badSpectra=NA)
{
   noID<-1;
   numNo<-length(scanNoList);
   noGroupID<-vector(mode='integer',length=numNo); noGroupID[]<-noID;
   if (is.na(badSpectra[1]))
   {
      #no bad spectra
      m<-1;
      while (m < numNo)
      {
          if ((scanNoList[m+1]-scanNoList[m])<=maxScanMissing)
          {
             noGroupID[m+1]<-noID;
          } else {
             noID<-noID+1;
             noGroupID[m+1]<-noID;
          }
          m<-m+1;
      }
   } else {
     badSpectra<-sort(badSpectra);
     m<-1;
     while (m<numNo)
     {
        indBad<-which(badSpectra<scanNoList[m+1] & badSpectra>scanNoList[m])
        if (length(indBad)>0)
        {
           noDiff<-max(scanNoList[m+1]-max(badSpectra[indBad]),min(badSpectra[indBad])-scanNoList[m]);
        } else {
           noDiff<-scanNoList[m+1]-scanNoList[m];
        }
        if (noDiff<=maxScanMissing)
        {
           noGroupID[m+1]<-noID;
        } else {
           noID<-noID+1;
           noGroupID[m+1]<-noID;
        }
        m<-m+1;
     }
   } 
   return(noGroupID);
}

