#################################################################
# This function is used to find bad spectra based on the Resolving Power variation
#      the scan numbers of bad spectra is saved to a file named badSpectraLog given by the user
# Usage:
#        pkGood<-removeBadSpectra(oriPeak,minRatio,badSpectraLog='badSpectraLog.csv',LargeRatioLog)
#                 
# Update:
#        March 5, 2012: first coded
#        March 7, 2012: remove the block to search previous two good spectra to compute the average Res
#                       using previous two instead
#################################################################
removeBadSpectra<-function(pklist,minRatio=0.2,badSpectraLog="badSpectraLog.csv",largeRatioLog=NA)
{
     isDebug<-FALSE;
     if (!is.na(largeRatioLog))
     {
        if (length(largeRatioLog)!=0) { isDebug=TRUE;}
     }
     # compute the resolving power: res=mz/PeakWidth
     resVect<-pklist$mz/pklist$Width;

     nTotlRec<-nrow(pklist);
     isRemove<-vector(mode='logical',length=nTotlRec);
     if (isDebug==TRUE)
     {
        resVectAve<-vector(mode='numeric',length=nTotlRec);
        resRatio<-vector(mode='numeric',length=nTotlRec);
        resRatioRev<-vector(mode='numeric',length=nTotlRec);
     }

     # compute the averaged resolving power[Res] for each scan number
     allScanNum<-pklist$No;
     scanNums<-sort(unique(allScanNum));
     nScanNum<-length(scanNums);
     aveRes<-vector(mode='numeric',length=nScanNum)
     for (nn in 1:nScanNum)
     {
          aveRes[nn]<-mean(resVect[which(allScanNum==scanNums[nn])]);
          if (isDebug==TRUE)
          {
             resVectAve[which(allScanNum==scanNums[nn])]<-aveRes[nn]
          } 
     }

     LargeRatioForward<-findLargeRatio(aveRes,nScanNum,minRatio,0)
     LargeRatioBackward<-findLargeRatio(aveRes,nScanNum,minRatio,1)

     isBadScanNum<-vector(mode='logical',length=nScanNum)
     isBadScanNum[which(LargeRatioBackward$isBadScanNum==TRUE & LargeRatioForward$isBadScanNum==TRUE)]<-TRUE;
     
     # ************  may check large resolving power [positive ratio]  ***************
     isBadScanNum[which(LargeRatioForward$resAveRatio>0)]<-FALSE;   #==> larger resolving power
     isBadScanNum[which(LargeRatioBackward$resAveRatio>0)]<-FALSE;  #==> larger resolving power

     # consider to do more fixing here [like mid-ratio, but seems not work well]

     if (isDebug==TRUE)
     {
        LargeRatioForward$scanNum<-scanNums;
        LargeRatioForward$aveRes<-aveRes;
        LargeRatioForward$RatioRev<-LargeRatioBackward$resAveRatio;
        LargeRatioForward$isScanBadRev<-LargeRatioBackward$isBadScanNum;
        LargeRatioForward$finalisBad<-isBadScanNum;
        write.table(LargeRatioForward,file=largeRatioLog,col.names=TRUE,sep=",",row.names=FALSE)
     }
     
     # check bad spectra
     for (nn in 1:nScanNum)
     {
         if (isBadScanNum[nn]==TRUE)
         {
            isRemove[which(allScanNum==scanNums[nn])]<-TRUE;
            if (isDebug==TRUE)
            {
               resRatio[which(allScanNum==scanNums[nn])] <-LargeRatioForward$resAveRatio[nn];
               resRatioRev[which(allScanNum==scanNums[nn])] <-LargeRatioBackward$resAveRatio[nn];
            }
         }
         if (isDebug==TRUE)
         {
            resRatio[which(allScanNum==scanNums[nn])] <-LargeRatioForward$resAveRatio[nn];
            resRatioRev[which(allScanNum==scanNums[nn])] <-LargeRatioBackward$resAveRatio[nn];
         }
     }

     # save the bad scan numbers
     badScanNums<-scanNums[isBadScanNum==TRUE];
     if (length(badScanNums)<1)
     {
       badScanNums <- 0;
     }
     write.table(badScanNums,file=badSpectraLog,col.names=FALSE,sep=",",row.names=FALSE);
  
     pklist$Width<-NULL;
     # return the values
     #pkGood<-pklist[which(isRemove==FALSE),];
     return(pklist[which(isRemove==FALSE),]);
}

#----------------------------------------------------------------------------------------------------
findLargeRatio<-function(aveRes,nScanNum,minRatio,opt=0)
{
   isBadScanNum<-vector(mode='logical',length=nScanNum);
   resAveRatio<-vector(mode='numeric',length=nScanNum);

   if (opt==1)
   {
      aveRes<-rev(aveRes);
   }
   for (nn in 1:nScanNum)
   {
       if (nn<3)
       {
          tmpAveRes<-(aveRes[(nn+1)]+aveRes[(nn+2)])*0.5;
          tmpRatio<-(aveRes[nn]-tmpAveRes)/tmpAveRes;
          if (abs(tmpRatio)>minRatio)
          {
             isBadScanNum[nn]<-TRUE;
          }
       } else {
         indBefore0<-nn-1;
         indBefore1<-indBefore0-1;
         tmpAveRes<-0.5*(aveRes[indBefore0]+aveRes[indBefore1]);
         tmpRatio<-(aveRes[nn]-tmpAveRes)/tmpAveRes;
         if (abs(tmpRatio)>minRatio)
         {
            isBadScanNum[nn]<-TRUE;
         }  
       }
       resAveRatio[nn]<-tmpRatio;
   }

   if (opt==1)
   {
     return(list(isBadScanNum=rev(isBadScanNum),resAveRatio=rev(resAveRatio)))
   } else {
     return(list(isBadScanNum=isBadScanNum,resAveRatio=resAveRatio))
   }
}
