####################################################
# remove dimer based on mz after findPeakPairs
# Usage:
#       peakNoDimer <-removeDimer(peaklist,mzTol)
#       peaklist: data frame/matrix contains scan-number[No],m/z [mz], intensity[maxo],rt
# Update:
#       Apr 21, 2013 :  coded the function
####################################################
removeDimer<-function(peaklist,mzTol=15e-6,debugfile=NA)
{

   #mz for H and Na
   mzH<-1.007276
   mzNa<-20.974668
   mzNa<-mzNa*(-1)
   nTop<-10

   # sort data by scan number, then mz_light
   peaklist<-as.data.frame(peaklist);
   peaklist.sorted <- peaklist[order(peaklist$scanNum,peaklist$mz_light,decreasing=FALSE), ]; rm(peaklist)
   
   peaklist.sorted$sample <- as.factor(peaklist.sorted$sample);
   samples<-levels(peaklist.sorted$sample);
   
   isDebug<-FALSE;
   if (length(debugfile)!=0 && !is.na(debugfile))  
   {
      isDebug<-TRUE;
   }

   indOri<-c(1:nrow(peaklist.sorted));
   isRemove<-vector(mode="integer",length=nrow(peaklist.sorted));

   for (s in 1:length(samples) )
   {
       indexSample<-which(peaklist.sorted$sample==samples[s]);
       currentIndexOri<-indOri[indexSample];

       # unique scan numbers within current sample
       scanNumVec<-peaklist.sorted$scanNum[currentIndexOri];
       scanNum<-unique(scanNumVec);
       numOfScanNum<-length(scanNum);

       # loop each scan-number to find the pairs
       for (nS in 1:numOfScanNum)
       {
           # get mz and int (light, heavy) for current scan number
           indexCurrentNo<-which(scanNumVec==scanNum[nS])
           mzLVecCurrentNo<-peaklist.sorted$mz_light[currentIndexOri[indexCurrentNo]]
           mzHVecCurrentNo<-peaklist.sorted$mz_heavy[currentIndexOri[indexCurrentNo]]
           intLVecCurrentNo<-peaklist.sorted$int_light[currentIndexOri[indexCurrentNo]]
           nPairs<-length(indexCurrentNo)
           # sort int_light, find the top three
           intSort<-sort(intLVecCurrentNo,decreasing=TRUE,index.return=TRUE);

           # check dimer for the top nTop highs in intensity

           for (n in 1:nTop)
           {
              if (n > nPairs) { break; }
              intC<-intSort$x[n]
              mzCLight<-mzLVecCurrentNo[intSort$ix[n]]
              mzCHeavy<-mzHVecCurrentNo[intSort$ix[n]]
              if (n<=3)
              {
                 # mix will all rest mz
                 if (n==nPairs)
                 {
                   cMZList<-c(mzCLight,0.5*(mzCLight+mzCHeavy))
                 } else {
                   cMZList<-getMZList(mzCLight,mzCHeavy,mzLVecCurrentNo[(n+1):nPairs],mzHVecCurrentNo[(n+1):nPairs])
                 } 
              } else {
                 # mix with rest top nTop [lower intensity ones]
                 if (n==nPairs || n==nTop)
                 {
                   cMZList<-c(mzCLight,0.5*(mzCLight+mzCHeavy))
                 } else {
                   cMZList<-getMZList(mzCLight,mzCHeavy,mzLVecCurrentNo[(n+1):nTop],mzHVecCurrentNo[(n+1):nTop])
                 }
              }
              numMZ<-length(cMZList)
              for (m in 1:numMZ)
              {
                 # remove H
                  indDimer<-findDimer(cMZList[m],mzH,mzLVecCurrentNo,mzTol)
                  if (length(indDimer)>0) 
                  { 
                     isRemove[currentIndexOri[indexCurrentNo[indDimer]]]<-TRUE;
                     if (isDebug==TRUE)
                     {
                       print("H: ");
                       print(peaklist.sorted[currentIndexOri[indexCurrentNo[indDimer]],])
                     }
                  }
                 # remove Na
                  indDimer<-findDimer(cMZList[m],mzNa,mzLVecCurrentNo,mzTol)
                  if (length(indDimer)>0) 
                  { 
                     isRemove[currentIndexOri[indexCurrentNo[indDimer]]]<-TRUE;
                     if (isDebug==TRUE)
                     {
                       print("Na: ");
                       print(peaklist.sorted[currentIndexOri[indexCurrentNo[indDimer]],])
                     }
                  }
              } # Loop all mixed mz list
         } # Loop for nTop
       } # scan number
   } # sample
   # return the pair matrix
   numRemove<-length(which(isRemove==TRUE))
   print(paste("removeDimer.R: a total of ",numRemove," pairs removed!",sep=""))
   return(peaklist.sorted[which(isRemove==FALSE),]);
}

getMZList<-function(mzALight,mzAHeavy,mzLightList,mzHeavyList)
{
   nLen<-length(mzLightList)
   mzList<-vector(mode="numeric",length=nLen*4+2)
   for (n in 1:nLen)
   {
      mzList[n]<-(mzLightList[n]+mzALight)*0.5  # current C12 and other C12 mixed
      mzList[nLen+n]<-(mzHeavyList[n]+mzALight)*0.5 # current C12 and other C13 mixed
      mzList[(2*nLen)+n]<-(mzLightList[n]+mzAHeavy)*0.5 # current C13 and other C12 mixed
      mzList[(3*nLen)+n]<-(mzHeavyList[n]+mzAHeavy)*0.5 # current C13 and other C13 mixed 
   }
   mzList[(4*nLen)+1]<-mzALight;   #C12
   mzList[(4*nLen)+2]<-(mzALight+mzAHeavy)*0.5; #current C12 C13 mixed
   return(unique(mzList))
}
#TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
findDimer<-function(mzC,mzD,mzList,mzTol)
{
   mzT<-2*mzC-mzD
   mzLow<-mzT/(1+mzTol)
   mzHigh<-mzT/(1-mzTol)
   # mzList>mzT & (mzList-mzT)<mzList*mzTol
   #   ==> mzList>mzT & mzList<mzT/(1-mzTol)
   # mzList<=mzT & (mzList-mzT)>-mzList*mzTol
   #   ==> mzList<=mzT & mzList>mzT/(1+mzTol)
   indDimer<-which(mzList>mzLow & mzList<mzHigh)
   return(indDimer)
}
