# function to remove homogeneous peaks following two steps
#          step 1: remove those lower intensity ones relative to the maximum within a specified mz window [mzWin]
#          step 2: based on the intensity slope keep only the peaks from the result of step 1
#                  [need set isFindPeak to be true, which is the default case]
# usage:
#         pkgood<-RemoveHomoPeaks(pklist,mzWin,mzSubWin,intSubRatio,maxMZDiff,isDebug)
# update:
#         Jan29, 2012   replaced data.frame with vector to speed up the code
#         Feb17, 2012   removed useless parameters
#         Feb18, 2012   changed the order of input parameters
#         Mar09, 2012   add sub-mz window and intensity ratios
#         Mar30, 2012   disable isFindPeak
removeHomoPeaks<-function(peaklist,mzWin=0.25, mzSubWin=c(0.0125,0.03,0.125),intSubRatio=c(0.1,0.01,0.0025),  maxMZDiff=0.1,isDebug=FALSE,isFindPeak=FALSE)
{
  # need No field from peaklist as the identifier of rt
  if (isFindPeak==TRUE)
  {
     print(paste("  removeHomoPeaks.R: maxMZDiff= ",maxMZDiff," mzWin=",mzWin,' two steps [mzWin and slope] are enabled'))
  }else {
     print(paste("  removeHomoPeaks.R: maxMZDiff= ",maxMZDiff," mzWin=",mzWin,' only mzWin is enabled'))
  }

  #peaklist$No<-as.factor(peaklist$No)
  nRT<-unique(peaklist$No)
  peaklist$oriIndex<-c(1:nrow(peaklist))
  isPeakVector<-vector(mode='logical',length=nrow(peaklist))

  nTotal<-length(nRT)

  if (isDebug==TRUE) print(paste("  Total scan number:",nTotal))

  for (numS in 1:nTotal)
  {
    # remove homogeneous peaks within specified small m_z window
    indcRT<-which(peaklist$No==nRT[numS])
    peaklist.ct<-peaklist[indcRT, ]
    peaklist.ctMZ<-peaklist.ct[order(peaklist.ct$mz,decreasing=F),] #sort by mz, ascending
    rm(peaklist.ct)

    oriIndxCT<-peaklist.ctMZ$oriIndex
    peaklist.ctMZ$indMZ<-c(1:nrow(peaklist.ctMZ))
    indmz0<-1
    tmpNrow<-nrow(peaklist.ctMZ)

    if (isDebug==TRUE) 
    {
       print(paste("numS:",numS, "ntoal:", nTotal," Removing HomoPeaks: done step 1 Time stamp: ",format(Sys.time(),"%Y-%B-%d %H:%M:%S")));
    }
    while (indmz0<=tmpNrow)
    {
          mz0<-peaklist.ctMZ$mz[indmz0]
          indWin <- which(peaklist.ctMZ$mz>=mz0 & peaklist.ctMZ$mz <= (mzWin+mz0))

          dataWin<-peaklist.ctMZ[indWin, ]
          
          IntMax<-max(dataWin$maxo)
          indMax<-which(dataWin$maxo==IntMax)
          indMax<-indMax[1];
          mzMax<-dataWin$mz[indMax]
          IntRatio<-dataWin$maxo/IntMax
          MZDiff<-abs(dataWin$mz-mzMax)
          isHighEnough<-vector(mode='logical',length=length(indWin));
          isHighEnough[MZDiff<mzSubWin[1] & IntRatio>intSubRatio[1]]<-TRUE; 
          
          isHighEnough[MZDiff>=mzSubWin[1] & MZDiff<mzSubWin[2]  & IntRatio>intSubRatio[2]]<-TRUE; 
          isHighEnough[MZDiff>=mzSubWin[2] & IntRatio>intSubRatio[3]]<-TRUE; 
          
          tmpIndex<-which(isHighEnough==TRUE)
          # save the tag of peaks
          isPeakVector[oriIndxCT[indWin[tmpIndex]]]<-TRUE
           
          indmz0<-indmz0+length(indWin)
    }
    rm(peaklist.ctMZ)

    # -----------------------
    #find good peaks within current scan number
    if (isFindPeak  )
    {
         indcRT<-which(peaklist$No==nRT[numS] & isPeakVector==TRUE)
         if (length(indcRT)>=3 )
         {
            dataValidSN<-peaklist[indcRT, ]
            rm(indcRT)
            #sort by mz
            dataValidSN.sorted<-dataValidSN[order(dataValidSN$mz,decreasing=F),]; rm(dataValidSN)
            tmpNrow<-nrow(dataValidSN.sorted)
            dataValidSN.sorted$slopeLeft<-(c(1:tmpNrow)*0);
            dataValidSN.sorted$slopeLeft[2:tmpNrow]<-dataValidSN.sorted$maxo[2:tmpNrow]/dataValidSN.sorted$maxo[1:(tmpNrow-1)]
            dataValidSN.sorted$slopeLeft[1]<-1
            dataValidSN.sorted$slopeRight<-0;
            dataValidSN.sorted$slopeRight[1:(tmpNrow-1)]<-dataValidSN.sorted$maxo[2:tmpNrow]/dataValidSN.sorted$maxo[1:(tmpNrow-1)]
            dataValidSN.sorted$slopeRight[tmpNrow]<-1;
            
            dataValidSN.sorted$mzDiffLeft<-0;
            dataValidSN.sorted$mzDiffLeft[2:tmpNrow]<-dataValidSN.sorted$mz[2:tmpNrow]-dataValidSN.sorted$mz[1:(tmpNrow-1)]
            dataValidSN.sorted$mzDiffLeft[1]<-maxMZDiff*2
            
            dataValidSN.sorted$mzDiffRight<-0;
            dataValidSN.sorted$mzDiffRight[1:(tmpNrow-1)]<-dataValidSN.sorted$mz[2:(tmpNrow)]-dataValidSN.sorted$mz[1:(tmpNrow-1)]
            dataValidSN.sorted$mzDiffRight[tmpNrow]<-maxMZDiff*2
            
            IndPeaks<-c(1:tmpNrow)*0;
            IndPeaksOri<-dataValidSN.sorted$oriIndex
           
            # find the possible peaks based on the slopes
            # should be  mzDiffRight & mzDiffLeft>maxMZDiff    |
            IndPeaks[which(dataValidSN.sorted$mzDiffLeft>=maxMZDiff & dataValidSN.sorted$mzDiffRight>=maxMZDiff)]<-1
            # should be left-slope>1 & mzDiffRight>maxMZDiff    . : |
            IndPeaks[which(dataValidSN.sorted$slopeLeft>1 & dataValidSN.sorted$mzDiffRight>=maxMZDiff)]<-1
            # should be right-slope<1 & mzDiffLeft>maxMZDiff    | : .
            IndPeaks[which(dataValidSN.sorted$slopeRight<1 & dataValidSN.sorted$mzDiffLeft>=maxMZDiff)]<-1
            # should be right-slope<1 & left-slope>1 & within MZ-range    : | :
            IndPeaks[which(dataValidSN.sorted$slopeRight<1 & dataValidSN.sorted$slopeLeft>1 & dataValidSN.sorted$mzDiffLeft<maxMZDiff & dataValidSN.sorted$mzDiffRight<maxMZDiff)]<-1
        
            IndPeaksOriNotFound<-IndPeaksOri[which(IndPeaks==0)]
            isPeakVector[IndPeaksOriNotFound]<-FALSE  
            rm(dataValidSN.sorted)
          } else if (length(indcRT)==2)
          {
            mz0<-peaklist$mz[indcRT[1]]
            mz1<-peaklist$mz[indcRT[2]]
            maxo0<-peaklist$maxo[indcRT[1]]
            maxo1<-peaklist$maxo[indcRT[2]]
            if (abs(mz0-mz1)>=maxMZDiff)
            {
               isPeakVector[indcRT]<-TRUE
            } else {
               if (maxo0>maxo1)
               {
                  isPeakVector[indcRT[2]]<-FALSE;
                  #peaklist$isPeak[indcRT[1]]<-2
               } else {
                  isPeakVector[indcRT[1]]<-FALSE;
                  #peaklist$isPeak[indcRT[2]]<-2
               }
            }
            
          } else if (length(indcRT)==1)
          {
             #peaklist$isPeak[indcRT]<-2
             isPeakVector[indcRT]<-TRUE
          }
    }
  }
  peaklist$oriIndex<-NULL;
  peaklist<-peaklist[which(isPeakVector==TRUE), ]
  return (peaklist)
}
