# function to remove tailing-peaks [ similar to homogeneous peaks]
# for each scan number, sorted by mz
#        find a peak with intesnity greater than intBase as the base-intensity,
#        remove those with intensity lower than 5% of the base-intensity within a mz window
#        mz window is defined as mzWin or to next mz with intensity greater than base-intensity
# usage:
#         pkgood<-removeTailing(pklist,intBase,intValidThreshold,mzWin,debugFile)
# update:
#         May19 2013    first code   
removeTailing<-function(peaklist,intBase=200,intValidThreshold=0.05, mzWin=1, debugFile=NA)
{
  print(paste("removeTailing: intBase=",intBase," intValidThreshold=",intValidThreshold," mzWin=",mzWin))

  scanNo<-unique(peaklist$No)
  oriIndex<-c(1:nrow(peaklist))
  isBadPeakVector<-vector(mode='logical',length=nrow(peaklist))

  nTotal<-length(scanNo)

  for (numS in 1:nTotal)
  {
    # extract peaks within a scan number, sorted by mz
    indcScanNo<-which(peaklist$No==scanNo[numS])
 
    oricMZ<-peaklist$mz[indcScanNo]
    tmpsort<-sort(oricMZ,index.return=TRUE)
    sortMZ<-tmpsort$x                             # sorted mz
    sortInt<-peaklist$maxo[indcScanNo[tmpsort$ix]] # corresponding intensity 
    sortOriIndex<-oriIndex[indcScanNo[tmpsort$ix]] # corresponding original index

    indIntBase<-which(sortInt>=intBase) # location of all intensity greater than intBase
    numMZ<-length(indcScanNo)

    if (numMZ==1 || length(indIntBase)<=1) # if only one found
    {
      next;
    }

    numHighInt<-length(indIntBase)

    for (n in 1:numHighInt)
    {
       ind0<-indIntBase[n]
       int0<-sortInt[ind0]
       mz0<-sortMZ[ind0]
       indMZWin<-which(sortMZ>=mz0 & sortMZ<mz0+mzWin)

       numMZWin<-length(indMZWin)
       if (n<numHighInt)
       {
          if (indMZWin[numMZWin]>indIntBase[n+1])
          {
             ind1<-indIntBase[n+1]-1;
          } else {
             ind1<-indMZWin[numMZWin]
          }
       } else {
         ind1<-indMZWin[numMZWin]
       }
       intMZWin<-sortInt[ind0:ind1]
       oriIndexMZWin<-sortOriIndex[ind0:ind1]
       indBadPeak<-which(intMZWin<intValidThreshold*int0)
       if (length(indBadPeak)>0) {isBadPeakVector[oriIndexMZWin[indBadPeak]]<-TRUE;}
    } # loop of high-int-peaks
  } # loop of scan number


  if (length(debugFile)!=0 && !is.na(debugFile))
  {
     peaklist$isBadPeak<-isBadPeakVector;
     write.table(peaklist,file=debugFile,col.name=TRUE,sep=",",row.name=FALSE);
     peaklist$isBadPeak<-NULL;
  }

  peaklist<-peaklist[which(isBadPeakVector==FALSE), ]
  return (peaklist)
}
